@@ -1,12 +1,13 @@
 //!
 //! # TT
 //!
-//! To be correct, To be reliable.
+//! **T**o be correct, **T**o be reliable.
 //!
-//! Lightweight private cloud solution for SME scenarios.
+//! Lightweight private-cloud solution for SME scenarios.
 //!
 
 #![warn(missing_docs, unused_import_braces, unused_extern_crates)]
 
 pub mod err;
+pub mod master;
 pub mod model;
